# Contact Form Plugin

A simple plugin for customisable contact forms on Kirby CMS powered pages.

## Setup

- Add the `contact-form` folder to your `site/plugins` folder.
- Include the plugin's JavaScript on every page that has one or more contact forms on it (`<?= js($kirby->url('media') . '/plugins/bearlydigital/contact-form/index.js') ?> `)
- `Optional`: Configure e-mail transport in your `config/config.php` to enable sending out e-mails on form submits. (see [Kirby Docs](https://getkirby.com/docs/guide/emails#transport-configuration))

## Creating forms: Blueprint

To create a contact form, add a `structure` type field to a page blueprint. The structure has to contain all the fields
you want to add to the contact form. The plugin will pull validation rules and general configuration from this field.
If you want to hide the field from the panel, simply set `hidden: true` on the structure. This will not impact the plugin
in any way.

### Blueprint notes:

You **have** to set the following value on the structure field in your blueprint: `smartcontact: true`, otherwise you will not be able to submit the form. This is a safe-guard to prevent people from using the contact forms code to change values on your website that they aren't supposed to be able to change.

Include the `mail` configuration in your blueprint, in order to send an e-mail every time your form is submitted. The configuration can be seen in the example below. Note that the `template` is loaded like a Kirby e-mail template ([see Kirby docs](https://getkirby.com/docs/guide/emails#plain-text)) and should always be located in `site/templates/emails`.

### Blueprint example:

```yaml
title: "Home"
fields:
  contactForm:
    type: structure
    label: Contact Form Submissions
    smartcontact: true
    mail:
      to: demo@bearly.test
      from: from@bearly.test
      subject: My contact form message
      template: contactform
    fields:
      name:
        type: text
        minlength: 3
        required: true
      email:
        type: email
        minlength: 3
        maxlength: 200
        required: true
        validate:
          email: true
      message:
        type: textarea
        maxlength: 200
        required: true
      acceptTOS:
        type: toggle
        label: Accepted TOS
        validate:
          accepted: true
      authorization:
        type: select
        options:
          yes: Yes
          no: No
        required: true
      options:
      type: tags
      options:
        - Option A
        - Option B
        - Option C
        - Option D
        - Option E
      required: true
      cv:
        type: files
        multiple: false
      role:
        type: select
        required: true
        options:
          first: "One option"
          second: "Another option"
          third: "Oh wow, yet more options"
```

## Creating forms: HTML

To create the HTML for a form, simply create a new snippet in your kirby instance that contains an HTML `form` tag with
all the `input` elements you need. There is very little "magic" going on with the HTML form, we just have the following
configuration attributes that can be applied to the `form` itself:

`data-smart="true"`: Required attribute. This lets the plugin's javascript know that it needs to bind to this form.

`data-target="home"`: Required attribute. This attribute needs to point to the ID / UUID of the page that the form's structure field lives on. In our example, the field lives on the `home` page.

`data-field="contactForm"`: Required attribute. This attribute needs to point to the name of the structure field on the parent page. In our example, the field is called `contactForm`.

`data-token="<?= $site->getHashChallenge() ?>"`: Required attribute. This provides the javascript with the hash-challenge for anti-spam.

`data-rules="<?= $site->rulesForField('home', 'contactForm') ?>"`: Optional attribute. This provides the javascript with all the rules for your contact form. Replace "home" and "contactForm" with the names of the page and field that the form is pointing to (data-target and data-field should have the exact same values).

`data-live="true"`: Optional attribute. If set to "true" the form will live-update the validation status of previously invalid forms.

`data-redirect="/success"`: Optional attribute. If set to a string the javascript will redirect the user to the given page once the form has successfully been submitted.

### HTML example

```html
<form
  action="/contact-form"
  method="post"
  data-smart="true"
  data-target="home"
  data-field="contactForm"
  data-live="true"
  data-redirect="/success"
  data-token="<?= $site->getHashChallenge() ?>"
  data-rules="<?= $site->rulesForField('home', 'contactForm') ?>"
>
  <label for="name"> Name: </label><input type="text" name="name" id="name" />
  <label for="email">E-Mai</label> l: <input type="email" name="email" id="" />
  <label for="message"> Text: </label
  ><textarea name="message" id="" cols="30" rows="10"></textarea>
  <input type="checkbox" name="acceptTOS" id="" /> Accept TOS
  <div class="radio-group" data-name="authorization">
    <input
      type="radio"
      name="authorization"
      id="authorization_yes"
      value="yes"
    /><label for="authorization_yes">Yes</label>
    <input
      type="radio"
      name="authorization"
      id="authorization_no"
      value="no"
    /><label for="authorization_no">No</label>
  </div>
  <select name="role" id="role">
    <option value="" selected>Choose</option>
    <option value="first">"One option"</option>
    <option value="second">"Another option"</option>
    <option value="third">"Oh wow, yet more options"</option>
  </select>
  <p>
    <b>Several Options to choose from:</b>
    <input type="checkbox" name="options" value="Option A" id="" /> Option A
    <input type="checkbox" name="options" value="Option B" id="" /> Option B
    <input type="checkbox" name="options" value="Option C" id="" /> Option C
    <input type="checkbox" name="options" value="Option D" id="" /> Option D
    <input type="checkbox" name="options" value="Option E" id="" /> Option E
  </p>
  <input type="submit" value="Submit" />
  <input type="file" name="cv" id="cv" />
  <p data-success="true" style="display:none">Great success</p>
</form>
```

### Special Case: Radio Groups

Radio Buttons are a special case because they have multiple `input` elements for the same value. Therefore, they need to be wrapped in a parent element that has the class `radio-group` applied to them.
Radio Buttons always have to map to a `select` field in the blueprint. Their `value` fields should align with the `option` keys in the blueprint.
See the `authorization` field in the blueprint & HTML example above for more information

### Special Case: Multiselects

Multiselect fields are a special case where a single field has multiple options that it can take on at the same time.
In the Kirby blueprint itself, that is best represented using the `tags` field type.
In the HTML, these fields are represented using multiple `<input type="checkbox">` fields that have the same `name` attribute.
See the `options` field in the blueprint & HTML example abouve for more information.

### Success messages

If you do not have `data-redirect` set on your form, then any item inside the form that contains the `data-success="true"` attribute will be set to `display: block` by the javascript after successfully submitting the form. This is an easy way for displaying a short message to the user after form submit.

## Validation

The plugin will validate incoming data via the [Kirby Validators API](https://getkirby.com/docs/reference/system/validators). Every validation set on the blueprint will be checked on the server before allowing the values through.
You can add validation rules to fields in the blueprint's structure field using the `validate` option or by setting values such as `max` or `maxlength` on the fields directly.

### Responding to validation in HTML

Every field with an invalid validation result will get the CSS class `has-error` set on its `input`. The same applies to any `label` that correlates to that field.

### Validating checkboxes

If you want to validate that a checkbox has been checked (for agreeing to privacy policies, etc.) you can simply add a `toggle` field to the blueprint and then add `validate: accepted` to it. This way, the form is only valid and can only be submitted if the corresponding `input type="checkbox"` is checked by the user.

## JavaScript Events

The following events are dispatched on the `form` and on the individual `input`s:

- validation-success
- validation-failed

These events are emitted everytime the validation engines evaluates one or more fields.

The `form` also emits a `success` event once its content has been submitted. You can bind to this event to trigger more
detailed UI updates than just showing a success message.
