<?php

namespace BearlyDigital\ContactForm;

use Kirby\Data\Data;
use Kirby\Http\Response;
use Kirby\Toolkit\Str;
use Kirby\Toolkit\V;

class ContactFormController {
    public static function routes(): array
    {
        return [
            [
                'pattern' => '/contact-form',
                'method' => 'POST',
                'action' => function() {
                    return ContactFormController::handlePost();
                }
            ]
        ];
    }

    public static function handlePost(): Response
    {
        // hashCash
        try {
            $hashValid = HashCash::validateToken(get("obscurity"));
        } catch (HashExpiredException $e) {
            return Response::json([
                "hashValid" => false,
                "valid" => true,
                "newHash" => HashCash::createChallenge("contact"),
            ]);
        }

        // Validation
        $target = page(get('target'));
        $targetField = $target->blueprint()->field(get('field'));
        $fields = $targetField['fields'];
        $data = get('fields');
        $validated = [];
        $errors = [];
        if (!isset($targetField['smartcontact'])) {
            // Protect fields from content injection
            return Response::json(['valid' => false, 'hashValid' => true, 'message' => 'guarded field', 'errors' => []]);
        }

        // Validation continued
        foreach(array_keys($fields) as $key) {
            if (!self::validateField($fields[$key], $data[$key])) {
                $errors[] = $key;
            }
        }

        if (count($errors)) {
            return Response::json(['valid' => false, 'hashValid' => true, 'errors' => $errors]);
        }


        // Store new entry
        $new = [];
        foreach(array_keys($fields) as $key) {
            // Special Case Handling for File Uploads (B64)
            if ($fields[$key]['type'] === 'files') {
                if (!isset($data[$key]) || empty ($data[$key])) {
                    continue;
                }
                $file = tmpfile();
                $encodedData =  substr($data[$key]['value'],strpos($data[$key]['value'],",")+1);
                $encodedData = str_replace(' ', '+', $encodedData);
                $decodedData = base64_decode($encodedData);
                fwrite($file, $decodedData);
                $path = stream_get_meta_data($file)['uri'];
                $parts = pathinfo($data[$key]['name']);
                $gname = $parts['filename'];
                $gExt = $parts['extension'];
                $fname = '';
                kirby()->impersonate('kirby', function() use ($path, $data, $key, $target, &$fname, $gname, $gExt) {
                    $f = $target->createFile([
                        'source' => $path,
                        'filename' => $gname . '-' . uuid() . '.' . $gExt
                    ]);
                    $fname = $f->filename();
                });
                $new[$key] = Data::encode([$fname], 'yaml');
            } else {
                $new[$key] = htmlspecialchars($data[$key] ?? '');
            }
        }

        // Update hashCash only after validation so re-validation doesn't take ages in JS
        HashCash::updateSession(true);

        // Insert into page
        foreach(array_keys($fields) as $key) {
            // Special Case Handling for File Uploads (B64)
            if ($fields[$key]['type'] === 'files') {
                if (!isset($data[$key]) || empty ($data[$key])) {
                    continue;
                }
                $file = tmpfile();
                $encodedData =  substr($data[$key]['value'],strpos($data[$key]['value'],",")+1);
                $encodedData = str_replace(' ', '+', $encodedData);
                $decodedData = base64_decode($encodedData);
                fwrite($file, $decodedData);
                $path = stream_get_meta_data($file)['uri'];
                $parts = pathinfo($data[$key]['name']);
                $gname = $parts['filename'];
                $gExt = $parts['extension'];
                $fname = '';
                kirby()->impersonate('kirby', function() use ($path, $data, $key, $target, &$fname, $gname, $gExt) {
                    $f = $target->createFile([
                        'source' => $path,
                        'filename' => $gname . '-' . uuid() . '.' . $gExt
                    ]);
                    $fname = $f->filename();
                });
                $new[$key] = Data::encode([$fname], 'yaml');
            } else {
                $new[$key] = htmlspecialchars($data[$key] ?? '');
            }
        }
        $fieldName = get('field');
        $targetData = $target->$fieldName()->toStructure()->toArray();
        $targetData[] = $new;
        kirby()->impersonate('kirby', function() use ($new, $target, $fieldName, $targetData) {
            $target = $target->update([
                $fieldName => Data::encode($targetData, 'yaml')
            ]);
        });

        // Mail (if configured)
        if (isset($targetField['mail'])) {
            $attachments =  [];
            foreach($fields as $key => $field) {
                 if ($field['type'] === 'files' && $field['mailAttach']) {
                    if (isset($new[$key])) {
                        $file = $target->file(Data::decode($new[$key], 'yaml')[0]);
                        $attachments[] = $file->realpath();
                    }
                }
            }
            kirby()->email([
                "to" => $targetField['mail']['to'],
                "from" => $targetField['mail']['from'],
                "template" => $targetField['mail']['template'],
                "subject" => Str::template($targetField['mail']['subject'], $new),
                "data" => $new,
                "attachments" => $attachments
            ]);
        }

        return Response::json(['valid' => true, 'hashValid' => true,]);
    }

    public static function getRulesForField($page, $field): array
    {
        $page = page($page);
        $fields = $page->blueprint()->field($field)['fields'];
        $rules = [];
        foreach($fields as $field => $values) {
            $rules[$field] = self::getRules($values);
        }
        return $rules;
    }

    private static function validateField($field, $value): bool
    {
        $rules = self::getRules($field);
        $valid = (V::value($value, $rules, [], false));
        if($valid === true) {
            return true;
        } else {
            return false;
        }
    }

    private static function getRules($field): array
    {
        $rules = [];
        if(isset($field['required']) && $field['required']) {
            $rules['required'] = $field['required'];
        }
        if(isset($field['max'])) {
            $rules['max'] = $field['max'];
        }
        if(isset($field['maxlength'])) {
            $rules['maxlength'] = $field['maxlength'];
        }
        if(isset($field['minlength'])) {
            $rules['minlength'] = $field['minlength'];
        }
        if(isset($field['min'])) {
            $rules['min'] = $field['min'];
        }
        if(isset($field['validate'])) {
            foreach($field['validate'] as $key => $value) {
                $rules[$key] = $value;
            }
        }
        return $rules;
    }
}
