"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.notify = exports.livereloadPlugin = void 0;
const crypto_1 = require("crypto");
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const server_1 = require("./server");
const clients = new Set();
const errorSources = new Map();
const outputHashes = new Map();
function livereloadPlugin(options = {}) {
    const { port = 53099, host = '127.0.0.1' } = options;
    const baseUrl = `http://${host}:${port}`;
    return {
        name: 'livereload-plugin',
        async setup(build) {
            const { absWorkingDir: basedir = process.cwd() } = build.initialOptions;
            const bannerTemplate = await fs_1.promises.readFile(require.resolve('../banner.js'), 'utf-8');
            const banner = bannerTemplate.replace(/{baseUrl}/g, baseUrl);
            (0, server_1.createLivereloadServer)({ basedir, host, port, onSSE: res => clients.add(res) });
            build.initialOptions.banner ??= {};
            if (build.initialOptions.banner.js) {
                build.initialOptions.banner.js += `;${banner}`;
            }
            else {
                build.initialOptions.banner.js = banner;
            }
            let fullReloadOnCssUpdates = options.fullReloadOnCssUpdates;
            if (!build.initialOptions.metafile && !options.fullReloadOnCssUpdates) {
                console.warn('[esbuild-plugin-livereload]: "metafile" option is disabled, so CSS updates will trigger a full reload');
                fullReloadOnCssUpdates = true;
            }
            if (!build.initialOptions.write && !options.fullReloadOnCssUpdates) {
                console.warn('[esbuild-plugin-livereload]: "write" option is disabled, so CSS updates will trigger a full reload');
                fullReloadOnCssUpdates = true;
            }
            build.onEnd(async (result) => {
                let cssUpdate = false;
                if (result.metafile && !fullReloadOnCssUpdates) {
                    const outputs = Object.keys(result.metafile.outputs).map(o => path_1.default.resolve(basedir, o));
                    for (const outputFile of outputs.filter(o => o.endsWith('.css'))) {
                        const prevHash = outputHashes.get(outputFile);
                        const hash = await calculateHash(outputFile);
                        if (prevHash !== hash) {
                            outputHashes.set(outputFile, hash);
                            cssUpdate = true;
                        }
                    }
                }
                notify('esbuild', {
                    warnings: result.warnings,
                    errors: result.errors,
                    cssUpdate,
                });
            });
        },
    };
}
exports.livereloadPlugin = livereloadPlugin;
/**
 * Notifies connected clients that errors or warnings occurred from
 * a given source. If there are no errors and the notification originates
 * from esbuild, the page will be sent a reload request.
 *
 * @param errorSource - Key to use when identifying these errors and warnings.
 *                      Previous results will be overwritten for the same `errorSource`.
 * @param msg - Object containing errors and warnings from the given source
 * @param connectedClients - Set of long-lived server responses representing
 *                           clients currently connected to the livereload
 *                           server. Only required if you are implementing your
 *                           own livereload server.
 */
function notify(errorSource, msg, connectedClients = clients) {
    errorSources.set(errorSource, msg);
    const values = Array.from(errorSources.values());
    const errors = values.flatMap(v => v.errors ?? []);
    const warnings = values.flatMap(v => v.warnings ?? []);
    const reloadCss = msg.cssUpdate;
    const reloadPage = !reloadCss && errorSource === 'esbuild' && errors.length === 0;
    const event = reloadCss
        ? 'event: reload-css\n'
        : reloadPage
            ? 'event: reload\n'
            : 'event: build-result\n';
    const data = `data: ${JSON.stringify({ warnings, errors })}\n\n`;
    connectedClients.forEach(res => {
        res.write(event);
        res.write(data);
    });
    if (reloadPage)
        connectedClients.clear();
}
exports.notify = notify;
async function calculateHash(filePath) {
    return new Promise((resolve, reject) => {
        const hash = (0, crypto_1.createHash)('md5');
        const stream = (0, fs_1.createReadStream)(filePath);
        stream.on('data', d => hash.update(d));
        stream.on('end', () => {
            resolve(hash.digest('base64'));
        });
        stream.on('error', reject);
    });
}
//# sourceMappingURL=livereload-plugin.js.map